@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2024 The OpenSSL Project Authors. All Rights Reserved.
+ * Copyright 2006-2023 The OpenSSL Project Authors. All Rights Reserved.
  *
  * Licensed under the Apache License 2.0 (the "License").  You may not use
  * this file except in compliance with the License.  You can obtain a copy
@@ -21,138 +21,117 @@
 #include <openssl/bn.h>
 
 /* Request nonce length, in bits (must be a multiple of 8). */
-#define NONCE_LENGTH 64
+#define NONCE_LENGTH            64
 
 /* Name of config entry that defines the OID file. */
-#define ENV_OID_FILE "oid_file"
+#define ENV_OID_FILE            "oid_file"
 
 /* Is |EXACTLY_ONE| of three pointers set? */
 #define EXACTLY_ONE(a, b, c) \
-    ((a && !b && !c) || (b && !a && !c) || (c && !a && !b))
+        (( a && !b && !c) || \
+         ( b && !a && !c) || \
+         ( c && !a && !b))
 
 static ASN1_OBJECT *txt2obj(const char *oid);
 static CONF *load_config_file(const char *configfile);
 
 /* Query related functions. */
 static int query_command(const char *data, const char *digest,
-    const EVP_MD *md, const char *policy, int no_nonce,
-    int cert, const char *in, const char *out, int text);
+                         const EVP_MD *md, const char *policy, int no_nonce,
+                         int cert, const char *in, const char *out, int text);
 static TS_REQ *create_query(BIO *data_bio, const char *digest, const EVP_MD *md,
-    const char *policy, int no_nonce, int cert);
+                            const char *policy, int no_nonce, int cert);
 static int create_digest(BIO *input, const char *digest,
-    const EVP_MD *md, unsigned char **md_value);
+                         const EVP_MD *md, unsigned char **md_value);
 static ASN1_INTEGER *create_nonce(int bits);
 
 /* Reply related functions. */
 static int reply_command(CONF *conf, const char *section, const char *engine,
-    const char *queryfile, const char *passin, const char *inkey,
-    const EVP_MD *md, const char *signer, const char *chain,
-    const char *policy, const char *in, int token_in,
-    const char *out, int token_out, int text);
+                         const char *queryfile, const char *passin, const char *inkey,
+                         const EVP_MD *md, const char *signer, const char *chain,
+                         const char *policy, const char *in, int token_in,
+                         const char *out, int token_out, int text);
 static TS_RESP *read_PKCS7(BIO *in_bio);
 static TS_RESP *create_response(CONF *conf, const char *section, const char *engine,
-    const char *queryfile, const char *passin,
-    const char *inkey, const EVP_MD *md, const char *signer,
-    const char *chain, const char *policy);
+                                const char *queryfile, const char *passin,
+                                const char *inkey, const EVP_MD *md, const char *signer,
+                                const char *chain, const char *policy);
 static ASN1_INTEGER *serial_cb(TS_RESP_CTX *ctx, void *data);
 static ASN1_INTEGER *next_serial(const char *serialfile);
 static int save_ts_serial(const char *serialfile, ASN1_INTEGER *serial);
 
 /* Verify related functions. */
 static int verify_command(const char *data, const char *digest, const char *queryfile,
-    const char *in, int token_in,
-    const char *CApath, const char *CAfile,
-    const char *CAstore,
-    char *untrusted, X509_VERIFY_PARAM *vpm);
+                          const char *in, int token_in,
+                          const char *CApath, const char *CAfile,
+                          const char *CAstore,
+                          char *untrusted, X509_VERIFY_PARAM *vpm);
 static TS_VERIFY_CTX *create_verify_ctx(const char *data, const char *digest,
-    const char *queryfile,
-    const char *CApath, const char *CAfile,
-    const char *CAstore,
-    char *untrusted,
-    X509_VERIFY_PARAM *vpm);
+                                        const char *queryfile,
+                                        const char *CApath, const char *CAfile,
+                                        const char *CAstore,
+                                        char *untrusted,
+                                        X509_VERIFY_PARAM *vpm);
 static X509_STORE *create_cert_store(const char *CApath, const char *CAfile,
-    const char *CAstore, X509_VERIFY_PARAM *vpm);
+                                     const char *CAstore, X509_VERIFY_PARAM *vpm);
 static int verify_cb(int ok, X509_STORE_CTX *ctx);
 
 typedef enum OPTION_choice {
     OPT_COMMON,
-    OPT_ENGINE,
-    OPT_CONFIG,
-    OPT_SECTION,
-    OPT_QUERY,
-    OPT_DATA,
-    OPT_DIGEST,
-    OPT_TSPOLICY,
-    OPT_NO_NONCE,
-    OPT_CERT,
-    OPT_IN,
-    OPT_TOKEN_IN,
-    OPT_OUT,
-    OPT_TOKEN_OUT,
-    OPT_TEXT,
-    OPT_REPLY,
-    OPT_QUERYFILE,
-    OPT_PASSIN,
-    OPT_INKEY,
-    OPT_SIGNER,
-    OPT_CHAIN,
-    OPT_VERIFY,
-    OPT_CAPATH,
-    OPT_CAFILE,
-    OPT_CASTORE,
-    OPT_UNTRUSTED,
-    OPT_MD,
-    OPT_V_ENUM,
-    OPT_R_ENUM,
-    OPT_PROV_ENUM
+    OPT_ENGINE, OPT_CONFIG, OPT_SECTION, OPT_QUERY, OPT_DATA,
+    OPT_DIGEST, OPT_TSPOLICY, OPT_NO_NONCE, OPT_CERT,
+    OPT_IN, OPT_TOKEN_IN, OPT_OUT, OPT_TOKEN_OUT, OPT_TEXT,
+    OPT_REPLY, OPT_QUERYFILE, OPT_PASSIN, OPT_INKEY, OPT_SIGNER,
+    OPT_CHAIN, OPT_VERIFY, OPT_CAPATH, OPT_CAFILE, OPT_CASTORE, OPT_UNTRUSTED,
+    OPT_MD, OPT_V_ENUM, OPT_R_ENUM, OPT_PROV_ENUM
 } OPTION_CHOICE;
 
 const OPTIONS ts_options[] = {
     OPT_SECTION("General"),
-    { "help", OPT_HELP, '-', "Display this summary" },
-    { "config", OPT_CONFIG, '<', "Configuration file" },
-    { "section", OPT_SECTION, 's', "Section to use within config file" },
+    {"help", OPT_HELP, '-', "Display this summary"},
+    {"config", OPT_CONFIG, '<', "Configuration file"},
+    {"section", OPT_SECTION, 's', "Section to use within config file"},
 #ifndef OPENSSL_NO_ENGINE
-    { "engine", OPT_ENGINE, 's', "Use engine, possibly a hardware device" },
+    {"engine", OPT_ENGINE, 's', "Use engine, possibly a hardware device"},
 #endif
-    { "inkey", OPT_INKEY, 's', "File with private key for reply" },
-    { "signer", OPT_SIGNER, 's', "Signer certificate file" },
-    { "chain", OPT_CHAIN, '<', "File with signer CA chain" },
-    { "CAfile", OPT_CAFILE, '<', "File with trusted CA certs" },
-    { "CApath", OPT_CAPATH, '/', "Path to trusted CA files" },
-    { "CAstore", OPT_CASTORE, ':', "URI to trusted CA store" },
-    { "untrusted", OPT_UNTRUSTED, '<', "Extra untrusted certs" },
-    { "token_in", OPT_TOKEN_IN, '-', "Input is a PKCS#7 file" },
-    { "token_out", OPT_TOKEN_OUT, '-', "Output is a PKCS#7 file" },
-    { "passin", OPT_PASSIN, 's', "Input file pass phrase source" },
-    { "", OPT_MD, '-', "Any supported digest" },
+    {"inkey", OPT_INKEY, 's', "File with private key for reply"},
+    {"signer", OPT_SIGNER, 's', "Signer certificate file"},
+    {"chain", OPT_CHAIN, '<', "File with signer CA chain"},
+    {"CAfile", OPT_CAFILE, '<', "File with trusted CA certs"},
+    {"CApath", OPT_CAPATH, '/', "Path to trusted CA files"},
+    {"CAstore", OPT_CASTORE, ':', "URI to trusted CA store"},
+    {"untrusted", OPT_UNTRUSTED, '<', "Extra untrusted certs"},
+    {"token_in", OPT_TOKEN_IN, '-', "Input is a PKCS#7 file"},
+    {"token_out", OPT_TOKEN_OUT, '-', "Output is a PKCS#7 file"},
+    {"passin", OPT_PASSIN, 's', "Input file pass phrase source"},
+    {"", OPT_MD, '-', "Any supported digest"},
 
     OPT_SECTION("Query"),
-    { "query", OPT_QUERY, '-', "Generate a TS query" },
-    { "data", OPT_DATA, '<', "File to hash" },
-    { "digest", OPT_DIGEST, 's', "Digest (as a hex string)" },
-    { "queryfile", OPT_QUERYFILE, '<', "File containing a TS query" },
-    { "cert", OPT_CERT, '-', "Put cert request into query" },
-    { "in", OPT_IN, '<', "Input file" },
+    {"query", OPT_QUERY, '-', "Generate a TS query"},
+    {"data", OPT_DATA, '<', "File to hash"},
+    {"digest", OPT_DIGEST, 's', "Digest (as a hex string)"},
+    {"queryfile", OPT_QUERYFILE, '<', "File containing a TS query"},
+    {"cert", OPT_CERT, '-', "Put cert request into query"},
+    {"in", OPT_IN, '<', "Input file"},
 
     OPT_SECTION("Verify"),
-    { "verify", OPT_VERIFY, '-', "Verify a TS response" },
-    { "reply", OPT_REPLY, '-', "Generate a TS reply" },
-    { "tspolicy", OPT_TSPOLICY, 's', "Policy OID to use" },
-    { "no_nonce", OPT_NO_NONCE, '-', "Do not include a nonce" },
-    { "out", OPT_OUT, '>', "Output file" },
-    { "text", OPT_TEXT, '-', "Output text (not DER)" },
+    {"verify", OPT_VERIFY, '-', "Verify a TS response"},
+    {"reply", OPT_REPLY, '-', "Generate a TS reply"},
+    {"tspolicy", OPT_TSPOLICY, 's', "Policy OID to use"},
+    {"no_nonce", OPT_NO_NONCE, '-', "Do not include a nonce"},
+    {"out", OPT_OUT, '>', "Output file"},
+    {"text", OPT_TEXT, '-', "Output text (not DER)"},
 
     OPT_R_OPTIONS,
     OPT_V_OPTIONS,
     OPT_PROV_OPTIONS,
-    { NULL }
+    {NULL}
 };
 
 /*
  * This command is so complex, special help is needed.
  */
-static char *opt_helplist[] = {
+static char* opt_helplist[] = {
     "",
     "Typical uses:",
     " openssl ts -query [-rand file...] [-config file] [-data file]",
@@ -202,12 +181,13 @@ int ts_main(int argc, char **argv)
     if ((vpm = X509_VERIFY_PARAM_new()) == NULL)
         goto end;
 
+    opt_set_unknown_name("digest");
     prog = opt_init(argc, argv, ts_options);
     while ((o = opt_next()) != OPT_EOF) {
         switch (o) {
         case OPT_EOF:
         case OPT_ERR:
-        opthelp:
+ opthelp:
             BIO_printf(bio_err, "%s: Use -help for summary.\n", prog);
             goto end;
         case OPT_HELP:
@@ -225,8 +205,10 @@ int ts_main(int argc, char **argv)
         case OPT_QUERY:
         case OPT_REPLY:
         case OPT_VERIFY:
-            if (mode != OPT_ERR)
+            if (mode != OPT_ERR) {
+                BIO_printf(bio_err, "%s: Must give only one of -query, -reply, or -verify\n", prog);
                 goto opthelp;
+            }
             mode = o;
             break;
         case OPT_DATA:
@@ -309,18 +291,20 @@ int ts_main(int argc, char **argv)
     }
 
     /* No extra arguments. */
-    argc = opt_num_rest();
-    if (argc != 0 || mode == OPT_ERR)
+    if (!opt_check_rest_arg(NULL))
         goto opthelp;
+    if (mode == OPT_ERR) {
+        BIO_printf(bio_err, "%s: Must give one of -query, -reply, or -verify\n", prog);
+        goto opthelp;
+    }
 
     if (!app_RAND_load())
         goto end;
 
-    if (digestname != NULL) {
-        if (!opt_md(digestname, &md))
-            goto opthelp;
-    }
-    if (mode == OPT_REPLY && passin && !app_passwd(passin, NULL, &password, NULL)) {
+    if (!opt_md(digestname, &md))
+        goto opthelp;
+    if (mode == OPT_REPLY && passin &&
+        !app_passwd(passin, NULL, &password, NULL)) {
         BIO_printf(bio_err, "Error getting password.\n");
         goto end;
     }
@@ -337,7 +321,7 @@ int ts_main(int argc, char **argv)
         if ((data != NULL) && (digest != NULL))
             goto opthelp;
         ret = !query_command(data, digest, md, policy, no_nonce, cert,
-            in, out, text);
+                             in, out, text);
     } else if (mode == OPT_REPLY) {
         if (vpmtouched)
             goto opthelp;
@@ -348,20 +332,20 @@ int ts_main(int argc, char **argv)
                 goto opthelp;
         }
         ret = !reply_command(conf, section, engine, queryfile,
-            password, inkey, md, signer, chain, policy,
-            in, token_in, out, token_out, text);
+                             password, inkey, md, signer, chain, policy,
+                             in, token_in, out, token_out, text);
 
     } else if (mode == OPT_VERIFY) {
         if ((in == NULL) || !EXACTLY_ONE(queryfile, data, digest))
             goto opthelp;
         ret = !verify_command(data, digest, queryfile, in, token_in,
-            CApath, CAfile, CAstore, untrusted,
-            vpmtouched ? vpm : NULL);
+                              CApath, CAfile, CAstore, untrusted,
+                              vpmtouched ? vpm : NULL);
     } else {
         goto opthelp;
     }
 
-end:
+ end:
     X509_VERIFY_PARAM_free(vpm);
     EVP_MD_free(md);
     NCONF_free(conf);
@@ -391,7 +375,7 @@ static CONF *load_config_file(const char *configfile)
         const char *p;
 
         BIO_printf(bio_err, "Using configuration from %s\n", configfile);
-        p = NCONF_get_string(conf, NULL, ENV_OID_FILE);
+        p = app_conf_try_string(conf, NULL, ENV_OID_FILE);
         if (p != NULL) {
             BIO *oid_bio = BIO_new_file(p, "r");
             if (!oid_bio)
@@ -400,8 +384,7 @@ static CONF *load_config_file(const char *configfile)
                 OBJ_create_objects(oid_bio);
                 BIO_free_all(oid_bio);
             }
-        } else
-            ERR_clear_error();
+        }
         if (!add_oid_section(conf))
             ERR_print_errors(bio_err);
     }
@@ -412,8 +395,8 @@ static CONF *load_config_file(const char *configfile)
  * Query-related method definitions.
  */
 static int query_command(const char *data, const char *digest, const EVP_MD *md,
-    const char *policy, int no_nonce,
-    int cert, const char *in, const char *out, int text)
+                         const char *policy, int no_nonce,
+                         int cert, const char *in, const char *out, int text)
 {
     int ret = 0;
     TS_REQ *query = NULL;
@@ -449,7 +432,7 @@ static int query_command(const char *data, const char *digest, const EVP_MD *md,
 
     ret = 1;
 
-end:
+ end:
     ERR_print_errors(bio_err);
     BIO_free_all(in_bio);
     BIO_free_all(data_bio);
@@ -459,7 +442,7 @@ end:
 }
 
 static TS_REQ *create_query(BIO *data_bio, const char *digest, const EVP_MD *md,
-    const char *policy, int no_nonce, int cert)
+                            const char *policy, int no_nonce, int cert)
 {
     int ret = 0;
     TS_REQ *ts_req = NULL;
@@ -507,7 +490,7 @@ static TS_REQ *create_query(BIO *data_bio, const char *digest, const EVP_MD *md,
         goto err;
 
     ret = 1;
-err:
+ err:
     if (!ret) {
         TS_REQ_free(ts_req);
         ts_req = NULL;
@@ -523,7 +506,7 @@ err:
 }
 
 static int create_digest(BIO *input, const char *digest, const EVP_MD *md,
-    unsigned char **md_value)
+                         unsigned char **md_value)
 {
     int md_value_len;
     int rv = 0;
@@ -555,19 +538,15 @@ static int create_digest(BIO *input, const char *digest, const EVP_MD *md,
 
         *md_value = OPENSSL_hexstr2buf(digest, &digest_len);
         if (*md_value == NULL || md_value_len != digest_len) {
+            OPENSSL_free(*md_value);
+            *md_value = NULL;
             BIO_printf(bio_err, "bad digest, %d bytes "
-                                "must be specified\n",
-                md_value_len);
-            goto err;
+                       "must be specified\n", md_value_len);
+            return 0;
         }
     }
     rv = md_value_len;
-err:
-    if (rv <= 0) {
-        OPENSSL_free(*md_value);
-        *md_value = NULL;
-        rv = 0;
-    }
+ err:
     EVP_MD_CTX_free(md_ctx);
     return rv;
 }
@@ -595,7 +574,7 @@ static ASN1_INTEGER *create_nonce(int bits)
     memcpy(nonce->data, buf + i, nonce->length);
     return nonce;
 
-err:
+ err:
     BIO_printf(bio_err, "could not create nonce\n");
     ASN1_INTEGER_free(nonce);
     return NULL;
@@ -606,10 +585,10 @@ err:
  */
 
 static int reply_command(CONF *conf, const char *section, const char *engine,
-    const char *queryfile, const char *passin, const char *inkey,
-    const EVP_MD *md, const char *signer, const char *chain,
-    const char *policy, const char *in, int token_in,
-    const char *out, int token_out, int text)
+                         const char *queryfile, const char *passin, const char *inkey,
+                         const EVP_MD *md, const char *signer, const char *chain,
+                         const char *policy, const char *in, int token_in,
+                         const char *out, int token_out, int text)
 {
     int ret = 0;
     TS_RESP *response = NULL;
@@ -629,7 +608,7 @@ static int reply_command(CONF *conf, const char *section, const char *engine,
         }
     } else {
         response = create_response(conf, section, engine, queryfile,
-            passin, inkey, md, signer, chain, policy);
+                                   passin, inkey, md, signer, chain, policy);
         if (response != NULL)
             BIO_printf(bio_err, "Response has been generated.\n");
         else
@@ -641,7 +620,7 @@ static int reply_command(CONF *conf, const char *section, const char *engine,
     /* Write response. */
     if (text) {
         if ((out_bio = bio_open_default(out, 'w', FORMAT_TEXT)) == NULL)
-            goto end;
+        goto end;
         if (token_out) {
             TS_TST_INFO *tst_info = TS_RESP_get_tst_info(response);
             if (!TS_TST_INFO_print_bio(out_bio, tst_info))
@@ -665,7 +644,7 @@ static int reply_command(CONF *conf, const char *section, const char *engine,
 
     ret = 1;
 
-end:
+ end:
     ERR_print_errors(bio_err);
     BIO_free_all(in_bio);
     BIO_free_all(query_bio);
@@ -698,11 +677,11 @@ static TS_RESP *read_PKCS7(BIO *in_bio)
     if (!TS_RESP_set_status_info(resp, si))
         goto end;
     TS_RESP_set_tst_info(resp, token, tst_info);
-    token = NULL; /* Ownership is lost. */
-    tst_info = NULL; /* Ownership is lost. */
+    token = NULL;               /* Ownership is lost. */
+    tst_info = NULL;            /* Ownership is lost. */
     ret = 1;
 
-end:
+ end:
     PKCS7_free(token);
     TS_TST_INFO_free(tst_info);
     if (!ret) {
@@ -714,9 +693,9 @@ end:
 }
 
 static TS_RESP *create_response(CONF *conf, const char *section, const char *engine,
-    const char *queryfile, const char *passin,
-    const char *inkey, const EVP_MD *md, const char *signer,
-    const char *chain, const char *policy)
+                                const char *queryfile, const char *passin,
+                                const char *inkey, const EVP_MD *md, const char *signer,
+                                const char *chain, const char *policy)
 {
     int ret = 0;
     TS_RESP *response = NULL;
@@ -746,7 +725,7 @@ static TS_RESP *create_response(CONF *conf, const char *section, const char *eng
         if (!TS_RESP_CTX_set_signer_digest(resp_ctx, md))
             goto end;
     } else if (!TS_CONF_set_signer_digest(conf, section, NULL, resp_ctx)) {
-        goto end;
+            goto end;
     }
 
     if (!TS_CONF_set_ess_cert_id_digest(conf, section, resp_ctx))
@@ -771,7 +750,7 @@ static TS_RESP *create_response(CONF *conf, const char *section, const char *eng
         goto end;
     ret = 1;
 
-end:
+ end:
     if (!ret) {
         TS_RESP_free(response);
         response = NULL;
@@ -788,8 +767,8 @@ static ASN1_INTEGER *serial_cb(TS_RESP_CTX *ctx, void *data)
 
     if (serial == NULL) {
         TS_RESP_CTX_set_status_info(ctx, TS_STATUS_REJECTION,
-            "Error during serial number "
-            "generation.");
+                                    "Error during serial number "
+                                    "generation.");
         TS_RESP_CTX_add_failure_info(ctx, TS_INFO_ADD_INFO_NOT_AVAILABLE);
     } else {
         save_ts_serial(serial_file, serial);
@@ -811,15 +790,14 @@ static ASN1_INTEGER *next_serial(const char *serialfile)
     if ((in = BIO_new_file(serialfile, "r")) == NULL) {
         ERR_clear_error();
         BIO_printf(bio_err, "Warning: could not open file %s for "
-                            "reading, using serial number: 1\n",
-            serialfile);
+                   "reading, using serial number: 1\n", serialfile);
         if (!ASN1_INTEGER_set(serial, 1))
             goto err;
     } else {
         char buf[1024];
         if (!a2i_ASN1_INTEGER(in, serial, buf, sizeof(buf))) {
             BIO_printf(bio_err, "unable to load number from %s\n",
-                serialfile);
+                       serialfile);
             goto err;
         }
         if ((bn = ASN1_INTEGER_to_BN(serial, NULL)) == NULL)
@@ -833,7 +811,7 @@ static ASN1_INTEGER *next_serial(const char *serialfile)
     }
     ret = 1;
 
-err:
+ err:
     if (!ret) {
         ASN1_INTEGER_free(serial);
         serial = NULL;
@@ -855,23 +833,24 @@ static int save_ts_serial(const char *serialfile, ASN1_INTEGER *serial)
     if (BIO_puts(out, "\n") <= 0)
         goto err;
     ret = 1;
-err:
+ err:
     if (!ret)
         BIO_printf(bio_err, "could not save serial number to %s\n",
-            serialfile);
+                   serialfile);
     BIO_free_all(out);
     return ret;
 }
 
+
 /*
  * Verify-related method definitions.
  */
 
 static int verify_command(const char *data, const char *digest, const char *queryfile,
-    const char *in, int token_in,
-    const char *CApath, const char *CAfile,
-    const char *CAstore, char *untrusted,
-    X509_VERIFY_PARAM *vpm)
+                          const char *in, int token_in,
+                          const char *CApath, const char *CAfile,
+                          const char *CAstore, char *untrusted,
+                          X509_VERIFY_PARAM *vpm)
 {
     BIO *in_bio = NULL;
     PKCS7 *token = NULL;
@@ -890,16 +869,15 @@ static int verify_command(const char *data, const char *digest, const char *quer
     }
 
     if ((verify_ctx = create_verify_ctx(data, digest, queryfile,
-             CApath, CAfile, CAstore, untrusted,
-             vpm))
-        == NULL)
+                                        CApath, CAfile, CAstore, untrusted,
+                                        vpm)) == NULL)
         goto end;
 
     ret = token_in
         ? TS_RESP_verify_token(verify_ctx, token)
         : TS_RESP_verify_response(verify_ctx, response);
 
-end:
+ end:
     printf("Verification: ");
     if (ret)
         printf("OK\n");
@@ -916,11 +894,11 @@ end:
 }
 
 static TS_VERIFY_CTX *create_verify_ctx(const char *data, const char *digest,
-    const char *queryfile,
-    const char *CApath, const char *CAfile,
-    const char *CAstore,
-    char *untrusted,
-    X509_VERIFY_PARAM *vpm)
+                                        const char *queryfile,
+                                        const char *CApath, const char *CAfile,
+                                        const char *CAstore,
+                                        char *untrusted,
+                                        X509_VERIFY_PARAM *vpm)
 {
     TS_VERIFY_CTX *ctx = NULL;
     STACK_OF(X509) *certs;
@@ -969,20 +947,20 @@ static TS_VERIFY_CTX *create_verify_ctx(const char *data, const char *digest,
 
     /* Initialising the X509_STORE object. */
     if (TS_VERIFY_CTX_set_store(ctx,
-            create_cert_store(CApath, CAfile, CAstore, vpm))
-        == NULL)
+                                create_cert_store(CApath, CAfile, CAstore, vpm))
+            == NULL)
         goto err;
 
     /* Loading any extra untrusted certificates. */
     if (untrusted != NULL) {
         certs = load_certs_multifile(untrusted, NULL, "extra untrusted certs",
-            vpm);
+                                     vpm);
         if (certs == NULL || TS_VERIFY_CTX_set_certs(ctx, certs) == NULL)
             goto err;
     }
     ret = 1;
 
-err:
+ err:
     if (!ret) {
         TS_VERIFY_CTX_free(ctx);
         ctx = NULL;
@@ -993,7 +971,7 @@ err:
 }
 
 static X509_STORE *create_cert_store(const char *CApath, const char *CAfile,
-    const char *CAstore, X509_VERIFY_PARAM *vpm)
+                                     const char *CAstore, X509_VERIFY_PARAM *vpm)
 {
     X509_STORE *cert_ctx = NULL;
     X509_LOOKUP *lookup = NULL;
@@ -1025,8 +1003,7 @@ static X509_STORE *create_cert_store(const char *CApath, const char *CAfile,
             goto err;
         }
         if (X509_LOOKUP_load_file_ex(lookup, CAfile, X509_FILETYPE_PEM, libctx,
-                propq)
-            <= 0) {
+                                      propq) <= 0) {
             BIO_printf(bio_err, "Error loading file %s\n", CAfile);
             goto err;
         }
@@ -1049,7 +1026,7 @@ static X509_STORE *create_cert_store(const char *CApath, const char *CAfile,
 
     return cert_ctx;
 
-err:
+ err:
     X509_STORE_free(cert_ctx);
     return NULL;
 }
