@@ -17,29 +17,103 @@
 #include <assert.h>
 #include <string.h>
 #include <openssl/crypto.h>
+#include <openssl/core_names.h>
+#include <openssl/proverr.h>
+#include <openssl/err.h>
+#include "internal/numbers.h"
 #include "blake2_impl.h"
 #include "prov/blake2.h"
 
-static const uint64_t blake2b_IV[8] = {
+static const OSSL_PARAM known_blake2b_ctx_params[] = {
+    {OSSL_DIGEST_PARAM_SIZE, OSSL_PARAM_UNSIGNED_INTEGER, NULL, 0, 0},
+    OSSL_PARAM_END
+};
+
+const OSSL_PARAM *ossl_blake2b_gettable_ctx_params(ossl_unused void *ctx,
+                                                   ossl_unused void *pctx)
+{
+    return known_blake2b_ctx_params;
+}
+
+const OSSL_PARAM *ossl_blake2b_settable_ctx_params(ossl_unused void *ctx,
+                                                   ossl_unused void *pctx)
+{
+    return known_blake2b_ctx_params;
+}
+
+int ossl_blake2b_get_ctx_params(void *vctx, OSSL_PARAM params[])
+{
+    struct blake2b_md_data_st *mdctx = vctx;
+    OSSL_PARAM *p;
+
+    BLAKE2B_CTX *ctx = &mdctx->ctx;
+
+    if (ctx == NULL)
+        return 0;
+    if (params == NULL)
+        return 1;
+
+    p = OSSL_PARAM_locate(params, OSSL_DIGEST_PARAM_SIZE);
+    if (p != NULL
+        && !OSSL_PARAM_set_uint(p, (unsigned int)mdctx->params.digest_length)) {
+        ERR_raise(ERR_LIB_PROV, PROV_R_FAILED_TO_SET_PARAMETER);
+        return 0;
+    }
+
+    return 1;
+}
+
+int ossl_blake2b_set_ctx_params(void *vctx, const OSSL_PARAM params[])
+{
+    size_t size;
+    struct blake2b_md_data_st *mdctx = vctx;
+    const OSSL_PARAM *p;
+
+    BLAKE2B_CTX *ctx = &mdctx->ctx;
+
+    if (ctx == NULL)
+        return 0;
+    if (params == NULL)
+        return 1;
+
+    p = OSSL_PARAM_locate_const(params, OSSL_DIGEST_PARAM_SIZE);
+    if (p != NULL) {
+        if (!OSSL_PARAM_get_size_t(p, &size)) {
+            ERR_raise(ERR_LIB_PROV, PROV_R_FAILED_TO_GET_PARAMETER);
+            return 0;
+        }
+        if (size < 1 || size > BLAKE2B_OUTBYTES) {
+            ERR_raise(ERR_LIB_PROV, PROV_R_INVALID_DIGEST_SIZE);
+            return 0;
+        }
+        ossl_blake2b_param_set_digest_length(&mdctx->params, (uint8_t)size);
+    }
+
+    return 1;
+}
+
+static const uint64_t blake2b_IV[8] =
+{
     0x6a09e667f3bcc908ULL, 0xbb67ae8584caa73bULL,
     0x3c6ef372fe94f82bULL, 0xa54ff53a5f1d36f1ULL,
     0x510e527fade682d1ULL, 0x9b05688c2b3e6c1fULL,
     0x1f83d9abfb41bd6bULL, 0x5be0cd19137e2179ULL
 };
 
-static const uint8_t blake2b_sigma[12][16] = {
-    { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 },
-    { 14, 10, 4, 8, 9, 15, 13, 6, 1, 12, 0, 2, 11, 7, 5, 3 },
-    { 11, 8, 12, 0, 5, 2, 15, 13, 10, 14, 3, 6, 7, 1, 9, 4 },
-    { 7, 9, 3, 1, 13, 12, 11, 14, 2, 6, 5, 10, 4, 0, 15, 8 },
-    { 9, 0, 5, 7, 2, 4, 10, 15, 14, 1, 11, 12, 6, 8, 3, 13 },
-    { 2, 12, 6, 10, 0, 11, 8, 3, 4, 13, 7, 5, 15, 14, 1, 9 },
-    { 12, 5, 1, 15, 14, 13, 4, 10, 0, 7, 6, 3, 9, 2, 8, 11 },
-    { 13, 11, 7, 14, 12, 1, 3, 9, 5, 0, 15, 4, 8, 6, 2, 10 },
-    { 6, 15, 14, 9, 11, 3, 0, 8, 12, 2, 13, 7, 1, 4, 10, 5 },
-    { 10, 2, 8, 4, 7, 6, 1, 5, 15, 11, 9, 14, 3, 12, 13, 0 },
-    { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 },
-    { 14, 10, 4, 8, 9, 15, 13, 6, 1, 12, 0, 2, 11, 7, 5, 3 }
+static const uint8_t blake2b_sigma[12][16] =
+{
+    {  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 } ,
+    { 14, 10,  4,  8,  9, 15, 13,  6,  1, 12,  0,  2, 11,  7,  5,  3 } ,
+    { 11,  8, 12,  0,  5,  2, 15, 13, 10, 14,  3,  6,  7,  1,  9,  4 } ,
+    {  7,  9,  3,  1, 13, 12, 11, 14,  2,  6,  5, 10,  4,  0, 15,  8 } ,
+    {  9,  0,  5,  7,  2,  4, 10, 15, 14,  1, 11, 12,  6,  8,  3, 13 } ,
+    {  2, 12,  6, 10,  0, 11,  8,  3,  4, 13,  7,  5, 15, 14,  1,  9 } ,
+    { 12,  5,  1, 15, 14, 13,  4, 10,  0,  7,  6,  3,  9,  2,  8, 11 } ,
+    { 13, 11,  7, 14, 12,  1,  3,  9,  5,  0, 15,  4,  8,  6,  2, 10 } ,
+    {  6, 15, 14,  9, 11,  3,  0,  8, 12,  2, 13,  7,  1,  4, 10,  5 } ,
+    { 10,  2,  8,  4,  7,  6,  1,  5, 15, 11,  9, 14,  3, 12, 13 , 0 } ,
+    {  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 } ,
+    { 14, 10,  4,  8,  9, 15, 13,  6,  1, 12,  0,  2, 11,  7,  5,  3 }
 };
 
 /* Set that it's the last block we'll compress */
@@ -81,15 +155,15 @@ static void blake2b_init_param(BLAKE2B_CTX *S, const BLAKE2B_PARAM *P)
 void ossl_blake2b_param_init(BLAKE2B_PARAM *P)
 {
     P->digest_length = BLAKE2B_DIGEST_LENGTH;
-    P->key_length = 0;
-    P->fanout = 1;
-    P->depth = 1;
+    P->key_length    = 0;
+    P->fanout        = 1;
+    P->depth         = 1;
     store32(P->leaf_length, 0);
     store64(P->node_offset, 0);
-    P->node_depth = 0;
-    P->inner_length = 0;
+    P->node_depth    = 0;
+    P->inner_length  = 0;
     memset(P->reserved, 0, sizeof(P->reserved));
-    memset(P->salt, 0, sizeof(P->salt));
+    memset(P->salt,     0, sizeof(P->salt));
     memset(P->personal, 0, sizeof(P->personal));
 }
 
@@ -104,14 +178,14 @@ void ossl_blake2b_param_set_key_length(BLAKE2B_PARAM *P, uint8_t keylen)
 }
 
 void ossl_blake2b_param_set_personal(BLAKE2B_PARAM *P, const uint8_t *personal,
-    size_t len)
+                                     size_t len)
 {
     memcpy(P->personal, personal, len);
     memset(P->personal + len, 0, BLAKE2B_PERSONALBYTES - len);
 }
 
 void ossl_blake2b_param_set_salt(BLAKE2B_PARAM *P, const uint8_t *salt,
-    size_t len)
+                                 size_t len)
 {
     memcpy(P->salt, salt, len);
     memset(P->salt + len, 0, BLAKE2B_SALTBYTES - len);
@@ -132,13 +206,13 @@ int ossl_blake2b_init(BLAKE2B_CTX *c, const BLAKE2B_PARAM *P)
  * Always returns 1.
  */
 int ossl_blake2b_init_key(BLAKE2B_CTX *c, const BLAKE2B_PARAM *P,
-    const void *key)
+                          const void *key)
 {
     blake2b_init_param(c, P);
 
     /* Pad the key to form first data block */
     {
-        uint8_t block[BLAKE2B_BLOCKBYTES] = { 0 };
+        uint8_t block[BLAKE2B_BLOCKBYTES] = {0};
 
         memcpy(block, key, P->key_length);
         ossl_blake2b_update(c, block, BLAKE2B_BLOCKBYTES);
@@ -150,8 +224,8 @@ int ossl_blake2b_init_key(BLAKE2B_CTX *c, const BLAKE2B_PARAM *P,
 
 /* Permute the state while xoring in the block of data. */
 static void blake2b_compress(BLAKE2B_CTX *S,
-    const uint8_t *blocks,
-    size_t len)
+                            const uint8_t *blocks,
+                            size_t len)
 {
     uint64_t m[16];
     uint64_t v[16];
@@ -192,36 +266,36 @@ static void blake2b_compress(BLAKE2B_CTX *S,
         S->t[0] += increment;
         S->t[1] += (S->t[0] < increment);
 
-        v[8] = blake2b_IV[0];
-        v[9] = blake2b_IV[1];
+        v[8]  = blake2b_IV[0];
+        v[9]  = blake2b_IV[1];
         v[10] = blake2b_IV[2];
         v[11] = blake2b_IV[3];
         v[12] = S->t[0] ^ blake2b_IV[4];
         v[13] = S->t[1] ^ blake2b_IV[5];
         v[14] = S->f[0] ^ blake2b_IV[6];
         v[15] = S->f[1] ^ blake2b_IV[7];
-#define G(r, i, a, b, c, d)                         \
-    do {                                            \
-        a = a + b + m[blake2b_sigma[r][2 * i + 0]]; \
-        d = rotr64(d ^ a, 32);                      \
-        c = c + d;                                  \
-        b = rotr64(b ^ c, 24);                      \
-        a = a + b + m[blake2b_sigma[r][2 * i + 1]]; \
-        d = rotr64(d ^ a, 16);                      \
-        c = c + d;                                  \
-        b = rotr64(b ^ c, 63);                      \
-    } while (0)
-#define ROUND(r)                           \
-    do {                                   \
-        G(r, 0, v[0], v[4], v[8], v[12]);  \
-        G(r, 1, v[1], v[5], v[9], v[13]);  \
-        G(r, 2, v[2], v[6], v[10], v[14]); \
-        G(r, 3, v[3], v[7], v[11], v[15]); \
-        G(r, 4, v[0], v[5], v[10], v[15]); \
-        G(r, 5, v[1], v[6], v[11], v[12]); \
-        G(r, 6, v[2], v[7], v[8], v[13]);  \
-        G(r, 7, v[3], v[4], v[9], v[14]);  \
-    } while (0)
+#define G(r,i,a,b,c,d) \
+        do { \
+            a = a + b + m[blake2b_sigma[r][2*i+0]]; \
+            d = rotr64(d ^ a, 32); \
+            c = c + d; \
+            b = rotr64(b ^ c, 24); \
+            a = a + b + m[blake2b_sigma[r][2*i+1]]; \
+            d = rotr64(d ^ a, 16); \
+            c = c + d; \
+            b = rotr64(b ^ c, 63); \
+        } while (0)
+#define ROUND(r)  \
+        do { \
+            G(r,0,v[ 0],v[ 4],v[ 8],v[12]); \
+            G(r,1,v[ 1],v[ 5],v[ 9],v[13]); \
+            G(r,2,v[ 2],v[ 6],v[10],v[14]); \
+            G(r,3,v[ 3],v[ 7],v[11],v[15]); \
+            G(r,4,v[ 0],v[ 5],v[10],v[15]); \
+            G(r,5,v[ 1],v[ 6],v[11],v[12]); \
+            G(r,6,v[ 2],v[ 7],v[ 8],v[13]); \
+            G(r,7,v[ 3],v[ 4],v[ 9],v[14]); \
+        } while (0)
 #if defined(OPENSSL_SMALL_FOOTPRINT)
         /* 3x size reduction on x86_64, almost 7x on ARMv8, 9x on ARMv4 */
         for (i = 0; i < 12; i++) {
@@ -303,7 +377,7 @@ int ossl_blake2b_update(BLAKE2B_CTX *c, const void *data, size_t datalen)
  */
 int ossl_blake2b_final(unsigned char *md, BLAKE2B_CTX *c)
 {
-    uint8_t outbuffer[BLAKE2B_OUTBYTES] = { 0 };
+    uint8_t outbuffer[BLAKE2B_OUTBYTES] = {0};
     uint8_t *target = outbuffer;
     int iter = (c->outlen + 7) / 8;
     int i;
